%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "scan.h"
int yyerror(const char *s);
%}

                    /* Definition section */

%term INT_CONST DOUBLE_CONST ENDL LPAR RPAR ILLEGAL
%term PLUS MINUS MULT DIV UMINUS

%left PLUS MINUS
%left MULT DIV
%nonassoc UMINUS    /* Fictive lexeme for unary minus */

%start f

%%
                    /* Grammar section */

                    /* Program is a sequence of lines containing expressions */
f :                 /* empty */
  | f e ENDL        { printf("= %lf\n", $2.double_value); }
  | f ENDL          { return 0; }           /* quit on empty line */
  | error ENDL      { yyerrok; }            /* error recovery */
  ;
                    /* Expression */
e : e PLUS e        {
                        $$.double_value = $1.double_value + $3.double_value;
                    }
  | e MINUS e       {
                        $$.double_value = $1.double_value - $3.double_value;
                    }
  | e MULT e        {
                        $$.double_value = $1.double_value * $3.double_value;
                    }
  | e DIV e         {
                        $$.double_value = $1.double_value / $3.double_value;
                    }
  | MINUS e %prec UMINUS { $$.double_value = (-$2.double_value); }
  | PLUS e          { $$ = $2; }
  | LPAR e RPAR     { $$ = $2; }
  | INT_CONST       { $$.double_value = (double) $1.int_value; }
  | DOUBLE_CONST    { $$ = $1; }
  ;

%%
                    /* Program section */

int main() {
    printf("Enter an expression to evaluate (empty line for quit):\n");
    yyparse();
    return 0;
}

/* Parse error diagnostics */
int yyerror(const char *s) {
    printf("%s\n", s);
    return 0;
}
